//
//  SceneDelegate.swift
//  Reader
//
//  Created by Shadowfacts on 10/29/21.
//

import UIKit
import OSLog

class SceneDelegate: UIResponder, UIWindowSceneDelegate {

    var window: UIWindow?

    private(set) var fervorController: FervorController!
    
    private let logger = Logger(subsystem: Bundle.main.bundleIdentifier!, category: "SceneDelegate")

    func scene(_ scene: UIScene, willConnectTo session: UISceneSession, options connectionOptions: UIScene.ConnectionOptions) {
        // Use this method to optionally configure and attach the UIWindow `window` to the provided UIWindowScene `scene`.
        // If using a storyboard, the `window` property will automatically be initialized and attached to the scene.
        // This delegate does not imply the connecting scene or session are new (see `application:configurationForConnectingSceneSession` instead).
        guard let windowScene = (scene as? UIWindowScene) else { return }
        
        window = UIWindow(windowScene: windowScene)
        window!.tintColor = .appTintColor
        
        if let account = LocalData.mostRecentAccount() {
            fervorController = FervorController(account: account)
            createAppUI()
        } else {
            let loginVC = LoginViewController()
            loginVC.delegate = self
            window!.rootViewController = loginVC
        }
        
        window!.makeKeyAndVisible()
    }

    func sceneDidDisconnect(_ scene: UIScene) {
        // Called as the scene is being released by the system.
        // This occurs shortly after the scene enters the background, or when its session is discarded.
        // Release any resources associated with this scene that can be re-created the next time the scene connects.
        // The scene may re-connect later, as its session was not necessarily discarded (see `application:didDiscardSceneSessions` instead).
    }

    func sceneDidBecomeActive(_ scene: UIScene) {
        // Called when the scene has moved from an inactive state to an active state.
        // Use this method to restart any tasks that were paused (or not yet started) when the scene was inactive.
    }

    func sceneWillResignActive(_ scene: UIScene) {
        // Called when the scene will move from an active state to an inactive state.
        // This may occur due to temporary interruptions (ex. an incoming phone call).
        
        Task(priority: .userInitiated) {
            do {
                try await self.fervorController?.syncReadToServer()
            } catch {
                logger.error("Unable to sync read state to server: \(error.localizedDescription, privacy: .public)")
            }
        }
    }

    func sceneWillEnterForeground(_ scene: UIScene) {
        // Called as the scene transitions from the background to the foreground.
        // Use this method to undo the changes made on entering the background.
    }

    func sceneDidEnterBackground(_ scene: UIScene) {
        // Called as the scene transitions from the foreground to the background.
        // Use this method to save data, release shared resources, and store enough scene-specific state information
        // to restore the scene back to its current state.
    }
    
    private func createAppUI() {
        let home = HomeViewController(fervorController: fervorController)
        home.delegate = self
        let nav = AppNavigationController(rootViewController: home)
        nav.navigationBar.prefersLargeTitles = true
        window!.rootViewController = nav
            
        Task(priority: .userInitiated) {
            do {
                try await self.fervorController.syncAll()
            } catch {
                logger.error("Unable to sync from server: \(error.localizedDescription, privacy: .public)")
            }
        }
    }

}

extension SceneDelegate: LoginViewControllerDelegate {
    func didLogin(with controller: FervorController) {
        let account = LocalData.Account(instanceURL: controller.instanceURL, clientID: controller.clientID!, clientSecret: controller.clientSecret!, accessToken: controller.accessToken!)
        LocalData.accounts.append(account)
        LocalData.mostRecentAccountID = account.id
        fervorController = FervorController(account: account)
        createAppUI()
    }
}

extension SceneDelegate: HomeViewControllerDelegate {
    func switchToAccount(_ account: LocalData.Account) {
        LocalData.mostRecentAccountID = account.id
        fervorController = FervorController(account: account)
        createAppUI()
    }
}
